from unittest import TestCase
from webassets.filter import register_filter
from webassets.test import TempEnvironmentHelper
from webassets_elm import Elm


register_filter(Elm)


class ElmFilterTestCase(TempEnvironmentHelper, TestCase):

    elm_code = 'import Html exposing (h1, text)\nmain = h1 [] [text "I<3elm"]'
    default_files = {'main.elm': elm_code}

    def setUp(self):
        super(ElmFilterTestCase, self).setup()

    def test_elm_filter(self):
        self.mkbundle('main.elm', filters='elm', output='app.js').build()
        self.assertIn('I<3elm', self.get('app.js'))



from os import remove
from sys import platform
from tempfile import NamedTemporaryFile, TemporaryFile
from webassets.filter import ExternalTool

__all__ = ('Elm')


class Elm(ExternalTool):
    """Elm filter for webassets.

    Requires the ``elm-make`` to be available from the system path.

    Supports the following external configuration:

    ELM_MAKE_BIN
        The path to the ``elm-make`` binary. If not set, assumes ``elm-make``
        is in the system path.

    """

    name = 'elm'
    options = {'binary': 'ELM_MAKE_BIN'}
    max_debug_level = None

    def input(self, _in, out, **kw):
        """
        Currently Elm does not write to stdout
        (https://github.com/elm-lang/elm-make/issues/39), so we need to write
        the compiled contents to a temporary file and then read it in order to
        output to stdout.
        """

        # create a temp file
        tmp = NamedTemporaryFile(suffix='.js', delete=False)
        tmp.close()  # close it so windows can read it

        # write to a temp file
        elm_make = self.binary or 'elm-make'
        source = kw['source_path']
        write_args = [elm_make, source, '--output', tmp.name, '--yes']
        with TemporaryFile(mode='w') as fake_write_obj:
            self.subprocess(write_args, fake_write_obj)

        # read the temp file
        cat_or_type = 'type' if platform == 'win32' else 'cat'
        read_args = [cat_or_type, tmp.name]
        self.subprocess(read_args, out)
        remove(tmp.name)



from setuptools import setup, find_packages

REPO_URL = 'https://github.com/cuducos/webassets-elm'

setup(
    author='Eduardo Cuducos',
    author_email='cuducos@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    description='Elm filter for webassets',
    install_requires=['webassets'],
    keywords=['elm', 'webassets', 'assets', 'flask', 'django'],
    license='MIT',
    long_description='Check `webassets-elm at GitHub <{}>`_.'.format(REPO_URL),
    name='webassets-elm',
    packages=find_packages(),
    test_suite='nose.collector',
    tests_require=['nose'],
    url=REPO_URL,
    version='0.1.4',
    zip_safe=False
)
